#!/usr/bin/env node

// src/index.ts
import inquirer2 from "inquirer";

// src/utils/validateAppName.ts
function validateAppName(input) {
  if (/^(?:@[a-z0-9-*~][a-z0-9-*._~]*\/)?[a-z0-9-~][a-z0-9-._~]*$/.test(input)) {
    return true;
  } else {
    return "App name must be lowercase, alphanumeric, and only use -, _, and @";
  }
}
var validateAppName_default = validateAppName;

// src/cli/readInput.ts
import chalk3 from "chalk";
import inquirer from "inquirer";
import path3 from "path";

// src/utils/getPackageManager.ts
function getPackageManager() {
  const userAgent = process.env.npm_config_user_agent;
  if (userAgent) {
    if (userAgent.includes("yarn")) {
      return "yarn";
    }
    if (userAgent.includes("npm")) {
      return "npm";
    }
    if (userAgent.includes("pnpm")) {
      return "pnpm";
    }
  }
  return "npm";
}
var getPackageManager_default = getPackageManager;

// src/utils/logger.ts
import chalk from "chalk";
var logger = {
  log: (...args) => {
    console.log(...args);
  },
  error(...args) {
    console.log(chalk.red(...args));
  },
  warn(...args) {
    console.log(chalk.yellow(...args));
  },
  info(...args) {
    console.log(chalk.cyan(...args));
  },
  success(...args) {
    console.log(chalk.green(...args));
  },
  higlight(...args) {
    console.log(chalk.bgBlack(...args));
  }
};
var logger_default = logger;

// src/cli/program.ts
import { Command } from "commander";

// src/constants.ts
import path from "path";
import { fileURLToPath } from "url";
var __filename;
try {
  __filename = fileURLToPath(import.meta.url);
} catch (error) {
  throw error;
}
var distPath = path.dirname(__filename);
var PKG_ROOT = path.join(distPath, "../");
var APP_NAME = "create-chakra";
var COMMON_TEMPLATES_ROOT = path.join(PKG_ROOT, "templates/common");

// package.json
var package_default = {
  name: "create-chakra",
  version: "0.0.5",
  author: "Vedant Nandwana",
  main: "./lib/index.js",
  type: "module",
  exports: "./dist/index.js",
  description: "\u2692 Tool to scaffold chakra apps, without wasting time.",
  keywords: [
    "chakra",
    "chakra-ui",
    "next",
    "react",
    "nextjs",
    "javascript",
    "typescript"
  ],
  bin: {
    "create-chakra": "./dist/index.js"
  },
  scripts: {
    build: "tsup src/index.ts --format esm --clean --sourcemap --minify --metafile",
    dev: 'tsup src/index.ts --format esm --watch --clean --onSuccess "node dist/index.js"',
    start: "node dist/index.js",
    test: 'echo "Error: no test specified" && exit 1',
    release: "standard-version",
    "publish:beta": "pnpm run build && npm publish --tag beta",
    publish: "pnpm run build && npm publish",
    lint: "eslint",
    prepare: "husky install",
    format: "prettier --write ./src"
  },
  devDependencies: {
    "@types/chalk": "^2.2.0",
    "@types/clear": "^0.1.2",
    "@types/commander": "^2.12.2",
    "@types/figlet": "^1.5.4",
    "@types/fs-extra": "^9.0.13",
    "@types/inquirer": "^9.0.0",
    "@types/node": "^18.6.4",
    "@types/yargs": "^17.0.11",
    eslint: "^8.21.0",
    husky: "^8.0.1",
    nodemon: "^2.0.19",
    prettier: "^2.7.1",
    "ts-node": "^10.9.1",
    tsup: "^6.2.1",
    typescript: "^4.7.4"
  },
  dependencies: {
    chalk: "4.1.2",
    clear: "^0.1.0",
    commander: "^9.4.0",
    figlet: "^1.5.2",
    "fs-extra": "^10.1.0",
    inquirer: "^9.1.0",
    ora: "^6.1.2",
    path: "^0.12.7",
    yargs: "^17.5.1"
  }
};

// src/utils/getVersion.ts
function getVersion() {
  return package_default.version;
}
var getVersion_default = getVersion;

// src/cli/program.ts
var program = new Command().name(APP_NAME);
program.description("A CLI for quickly creating applications based on Chakra UI").argument("[app]", "The name of the application").option("--template <templateId>", "The template to use").version(getVersion_default()).parse(process.argv);
var program_default = program;

// src/utils/resolvePackageManager.ts
function resolvePacakgeManager(packageManager) {
  return packageManager === "npm" ? "npx --yes" : packageManager;
}
var resolvePackageManager_default = resolvePacakgeManager;

// src/cli/commands/createReact.ts
function createReactCommand(input) {
  const { packageManager, appConfig, projectName } = input;
  const parts = [resolvePackageManager_default(packageManager)];
  if (packageManager === "npm") {
    parts.push("create-vite@latest");
  } else {
    parts.push("create vite");
  }
  parts.push(projectName);
  parts.push("--template", appConfig.templateId);
  return parts.join(" ");
}
var createReact_default = createReactCommand;

// src/cli/commands/createNext.ts
function createNextCommand(input) {
  const { appConfig, packageManager } = input;
  const parts = [resolvePackageManager_default(packageManager)];
  if (packageManager === "npm") {
    parts.push("create-next-app@latest");
  } else {
    parts.push("create next-app");
  }
  parts.push(input.projectName);
  if (appConfig.language === "ts") {
    if (packageManager === "npm" || packageManager === "pnpm") {
      parts.push("--ts");
    } else {
      parts.push("--typescript");
    }
  }
  return parts.join(" ");
}
var createNext_default = createNextCommand;

// src/cli/config.ts
import fs from "fs-extra";
import path2 from "path";
import chalk2 from "chalk";
var supportedDependencies = ["prettier", "clsx"];
var supportedTemplateIds = [
  "nextjs",
  "nextjs-ts",
  "react",
  "react-ts"
];
var NEXTJS_CONFIG = {
  templateId: "nextjs",
  displayName: `Next.js ${chalk2.dim("(create-next-app)")}`,
  language: "js",
  templateDir: path2.join(PKG_ROOT, "templates/nextjs"),
  scaffoldingTool: "create-next-app",
  twConfigExtension: ".js",
  copyTemplate: async ({ projectDir }) => {
    await fs.copy(
      path2.join(NEXTJS_CONFIG.templateDir, "index.jsx"),
      path2.join(projectDir, "pages", "index.js")
    );
    await fs.copy(
      path2.join(NEXTJS_CONFIG.templateDir, "_app.jsx"),
      path2.join(projectDir, "pages", "_app.js")
    );
  },
  deleteFiles: async ({ projectDir }) => {
    await fs.remove(path2.join(projectDir, "styles/Home.module.css"));
  },
  getCssOutputPath: ({ projectDir }) => {
    return path2.join(projectDir, "styles", "globals.css");
  },
  createInstallCommand: createNext_default
};
var NEXTJS_TS_CONFIG = {
  templateId: "nextjs-ts",
  displayName: `${chalk2.bold("Next.js TS")} ${chalk2.dim("(create-next-app)")}`,
  language: "ts",
  templateDir: path2.join(PKG_ROOT, "templates/nextjs-ts"),
  scaffoldingTool: "create-next-app",
  twConfigExtension: ".js",
  copyTemplate: async ({ projectDir }) => {
    await fs.copy(
      path2.join(NEXTJS_TS_CONFIG.templateDir, "index.tsx"),
      path2.join(projectDir, "pages", "index.tsx")
    );
    await fs.copy(
      path2.join(NEXTJS_CONFIG.templateDir, "_app.tsx"),
      path2.join(projectDir, "pages", "_app.tsx")
    );
  },
  deleteFiles: async ({ projectDir }) => {
    await fs.remove(path2.join(projectDir, "styles/Home.module.css"));
  },
  getCssOutputPath: ({ projectDir }) => {
    return path2.join(projectDir, "styles", "globals.css");
  },
  createInstallCommand: createNext_default
};
var REACT_CONFIG = {
  templateId: "react",
  displayName: `React ${chalk2.dim("(create-vite)")}`,
  language: "js",
  templateDir: path2.join(PKG_ROOT, "templates/react"),
  scaffoldingTool: "create-vite",
  twConfigExtension: ".cjs",
  copyTemplate: async ({ projectDir }) => {
    await fs.copy(
      path2.join(REACT_CONFIG.templateDir, "App.jsx"),
      path2.join(projectDir, "src/App.jsx")
    );
  },
  getCssOutputPath: ({ projectDir }) => {
    return path2.join(projectDir, "src/index.css");
  },
  deleteFiles: async ({ projectDir }) => {
    await fs.remove(path2.join(projectDir, "src/app.css"));
  },
  createInstallCommand: createReact_default
};
var REACT_TS_CONFIG = {
  templateId: "react-ts",
  displayName: `React ${chalk2.dim("(TypeScript, create-vite)")}`,
  language: "js",
  templateDir: path2.join(PKG_ROOT, "templates/react-ts"),
  scaffoldingTool: "create-vite",
  twConfigExtension: ".cjs",
  copyTemplate: async ({ projectDir }) => {
    await fs.copy(
      path2.join(REACT_TS_CONFIG.templateDir, "App.tsx"),
      path2.join(projectDir, "src/App.tsx")
    );
  },
  getCssOutputPath: ({ projectDir }) => {
    return path2.join(projectDir, "src/index.css");
  },
  deleteFiles: async ({ projectDir }) => {
    await fs.remove(path2.join(projectDir, "src/app.css"));
  },
  createInstallCommand: createReact_default
};
var CONFIG_BY_ID = {
  nextjs: NEXTJS_CONFIG,
  "nextjs-ts": NEXTJS_TS_CONFIG,
  react: REACT_CONFIG,
  "react-ts": REACT_TS_CONFIG
};
var getConfig = (configId) => CONFIG_BY_ID[configId];

// src/cli/readInput.ts
var DEFAULTS = {
  projectName: "chakra-app",
  dependencies: [],
  packageManager: getPackageManager_default(),
  projectDir: "",
  appConfig: NEXTJS_CONFIG
};
async function readInput() {
  const input = { ...DEFAULTS };
  const { template: templateId } = program_default.opts();
  input.projectName = program_default.args[0] ?? await readProjectName();
  if (await checkTemplateSupport(templateId)) {
    const config = getConfig(templateId);
    if (!config) {
      throw new Error(`Unknown template id: ${templateId}`);
    }
    input.appConfig = config;
  } else {
    const tid = await readTemplateId(
      supportedTemplateIds.filter((id) => !id.includes("ts"))
    );
    console.log("tid", tid);
    const language = await readLanguage();
    const templateIdKey = `${tid}${language === "ts" ? "-ts" : ""}`;
    console.log("templateIdKey", templateIdKey);
    const config = getConfig(templateIdKey);
    if (!config) {
      throw new Error(`Unknown template id: ${templateIdKey}`);
    }
    input.appConfig = config;
  }
  input.dependencies = await readDependencies();
  input.projectDir = path3.resolve(process.cwd(), input.projectName);
  return input;
}
async function readProjectName() {
  const { projectName } = await inquirer.prompt(
    {
      name: "projectName",
      type: "input",
      message: "Project name",
      default: DEFAULTS.projectName,
      validate: validateAppName_default,
      transformer: (i) => {
        return i.trim();
      }
    }
  );
  return projectName;
}
async function readTemplateId(types) {
  const { templateId } = await inquirer.prompt({
    name: "templateId",
    type: "list",
    message: "App type",
    choices: types.map((t) => {
      var _a;
      return {
        name: (_a = getConfig(t)) == null ? void 0 : _a.displayName,
        value: t
      };
    }),
    default: "nextjs"
  });
  return templateId;
}
async function readDependencies() {
  const { dependencies } = await inquirer.prompt({
    name: "dependencies",
    type: "checkbox",
    message: "Which dependencies would you like to include?",
    choices: supportedDependencies.map((dependency) => ({
      name: dependency,
      checked: false
    }))
  });
  return dependencies;
}
async function readLanguage() {
  const { language } = await inquirer.prompt({
    name: "language",
    type: "list",
    message: "What language will your project be written in?",
    choices: [
      { name: "TypeScript", value: "ts", short: "ts" },
      { name: "JavaScript", value: "js", short: "js" }
    ],
    default: "typescript"
  });
  return language;
}
async function checkTemplateSupport(templateId) {
  const templateSupported = supportedTemplateIds.includes(
    templateId
  );
  if (templateId && !templateSupported) {
    logger_default.error(`Unknown template: ${templateId}
`);
    logger_default.info(
      `Currently supported templates:
${chalk3.green(
        supportedTemplateIds.join("\n")
      )}`
    );
    logger_default.info(
      `You can skip passing the template and select it interactively.
`
    );
    const answer = await inquirer.prompt({
      name: "continue",
      type: "confirm",
      message: "Would you like to continue with interactive mode?"
    });
    if (!answer.continue) {
      process.exit(1);
    }
  }
  return templateSupported;
}
var readInput_default = readInput;

// src/index.ts
import chalk5 from "chalk";
import fs3 from "fs-extra";

// src/cli/output/installDependencies.ts
import fs2 from "fs-extra";
import ora from "ora";
import path4 from "path";

// src/utils/execAsync.ts
import { exec } from "child_process";
import { promisify } from "util";
var execa = promisify(exec);
var execAsync_default = execa;

// src/utils/installPackages.ts
async function installPackages(options) {
  const { packageManager, dev, projectDir, packages } = options;
  if (!packages.length) {
    return;
  }
  const installCommand = packageManager === "npm" ? "install" : "add";
  const cmd = `${packageManager} ${installCommand} ${packages.join(
    " "
  )}`;
  await execAsync_default(cmd, { cwd: projectDir });
}
var installPackages_default = installPackages;

// src/cli/output/installDependencies.ts
async function installDependencies(input) {
  const { projectDir } = input;
  const devDependencies = input.dependencies.filter(
    (d) => dependenciesMap[d] === "dev"
  );
  const dependencies = input.dependencies.filter(
    (d) => dependenciesMap[d] === "dependencies"
  );
  const packages = [
    "@chakra-ui/react",
    "@emotion/react",
    "@emotion/styled",
    "framer-motion",
    ...devDependencies
  ];
  const spinner = ora(`Installing dependencies`).start();
  await installPackages_default({
    dev: true,
    projectDir,
    packageManager: getPackageManager_default(),
    packages
  });
  await installPackages_default({
    dev: false,
    projectDir,
    packageManager: getPackageManager_default(),
    packages: dependencies
  });
  spinner.succeed(`Dependencies installed`);
  if (devDependencies.includes("prettier")) {
    const rc = path4.join(COMMON_TEMPLATES_ROOT, ".prettierrc");
    const ignore = path4.join(COMMON_TEMPLATES_ROOT, ".prettierignore");
    spinner.start(`Creating .prettierrc and .prettierignore`).start();
    await fs2.copy(rc, path4.join(projectDir, ".prettierrc"));
    await fs2.copy(ignore, path4.join(projectDir, ".prettierignore"));
    spinner.succeed(`.prettierrc and .prettierignore created`);
  }
}
var dependenciesMap = {
  prettier: "dev",
  clsx: "dependencies"
};

// src/index.ts
import figlet from "figlet";

// src/cli/output/createProject.ts
import { spawn } from "child_process";
import chalk4 from "chalk";
async function createProject(input) {
  const { appConfig } = input;
  const command = appConfig.createInstallCommand(input);
  logger_default.info(
    `
Installing project using ${chalk4.green(
      input.appConfig.scaffoldingTool
    )}
`
  );
  if (typeof command === "string") {
    const child = spawn(command, {
      stdio: "inherit",
      shell: true
    });
    await new Promise((resolve, reject) => {
      child.on("error", reject);
      child.on("close", (code) => {
        resolve(code);
      });
    });
  }
  if (typeof command === "function") {
    await command();
  }
  logger_default.log(
    `${chalk4.bold.green("\u2714")} Project created using ${chalk4.green.bold(
      appConfig.scaffoldingTool
    )}`
  );
}
var createProject_default = createProject;

// src/cli/output/installChakra.ts
import ora2 from "ora";
async function installChakra(input) {
  if (input.appConfig.skipChakraInstall) {
    return;
  }
  await copyChakraTemplate(input);
  await deleteFiles(input);
}
async function copyChakraTemplate(input) {
  const { appConfig } = input;
  const spinner = ora2(`Copying Chakra template`).start();
  await appConfig.copyTemplate(input);
  spinner.succeed(`Added Chakra template`);
}
async function deleteFiles(input) {
  var _a, _b;
  const spinner = ora2(`Deleting unneeded files`).start();
  await ((_b = (_a = input.appConfig).deleteFiles) == null ? void 0 : _b.call(_a, input));
  spinner.succeed("Deleted unneeded files");
}
var installChakra_default = installChakra;

// src/index.ts
import clear from "clear";
async function main() {
  clear();
  logger_default.info("\n");
  logger_default.higlight(
    figlet.textSync("create chakra", {
      font: "Mini"
    })
  );
  logger_default.info(`
${chalk5.bold("Welcome to create-chakra!")}`);
  logger_default.success("The easiest way to create a Chakra project\n");
  const input = await readInput_default();
  const { projectDir, projectName } = input;
  const pkgManager = getPackageManager_default();
  logger_default.info(`
Using: ${chalk5.cyan.bold(pkgManager)}
`);
  if (fs3.existsSync(projectDir)) {
    const answer = await inquirer2.prompt({
      name: "overwrite",
      type: "confirm",
      message: `${chalk5.yellow.bold(`Directory already exists. Overwrite?`)}`
    });
    if (!answer.overwrite) {
      logger_default.error("Aborting...");
      process.exit(1);
    }
    fs3.removeSync(projectDir);
  }
  await createProject_default(input);
  await installChakra_default(input);
  await installDependencies(input);
  logger_default.info(`
Project created in ${chalk5.green.bold(projectDir)}
`);
  logger_default.info(`${chalk5.cyan.bold(`cd ${projectName}`)}`);
  logger_default.info(
    `${chalk5.cyan.bold(
      `${getPackageManager_default()} ${getPackageManager_default() === "npm" ? "run" : ""} dev`
    )}
`
  );
  logger_default.log("Happy hacking!");
  process.exit(0);
}
main().catch((e) => {
  logger_default.error(`
${e}
`);
  process.exit(1);
});
